/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */


var savefileto =
{
    types: ["image", "link", "page"],
    
    showType:
    {
        get image() { return (gContextMenu.onLoadedImage || gContextMenu.onCanvas); },
        get link() { return gContextMenu.onSaveableLink; },
        get page() { return (! (gContextMenu.onTextInput || gContextMenu.onLink || gContextMenu.isContentSelected || gContextMenu.onImage || gContextMenu.onCanvas || gContextMenu.onVideo || gContextMenu.onAudio)); }
    },
    
    onLoad: function()
    {
        Cu.import("chrome://savefileto-modules/content/constants.jsm", this);

        // load os specific default preferences
        Cu.import("chrome://savefileto-modules/content/default-preferences.jsm", this);
        var prefsPath = "chrome://savefileto-platform/content/savefileto.js";
        this.loadDefaultPrefs(prefsPath);
        
        var self = this;

        Cu.import("chrome://savefileto-modules/content/savefileto.jsm", this);

        var prefs = this.savefileto.prefs;
        
        this.types.forEach(type => {
            // init access key
            this._updateMenuItemAccessKey(type);
            
            // init cascade menu
            let commandCallback = (event, folder) => {
                var prompt = (event.ctrlKey || prefs.prompt);
                window["savefileto_save_" + type].call(this, folder, prompt);
                this.savefileto.addToRecent(type, folder, window);
            }
            
            let cascadeMenu = this.savefileto.createCascadeMenu(window, type, commandCallback);
            cascadeMenu.attach(document.getElementById( "savefileto-context-save" + type));
        });
        
        // observe access key changes
        this.accessKeyChangeListener = function(prefs, propName)
        {
            var match = /^(page|link|image)AccessKey$/.exec(propName);
            if (match)
                self._updateMenuItemAccessKey(match[1]);
        };
        prefs.addListener(this.accessKeyChangeListener);

        // show/hide menu item
        let contextMenu = document.getElementById("contentAreaContextMenu");

        contextMenu.addEventListener('popupshowing', () => {
          savefileto.types.forEach(type => {
            document.getElementById('savefileto-context-save' + type).hidden = !(prefs[type + 'Enabled'] && savefileto.showType[type]);
          });
        });

        contextMenu.addEventListener('popuphidden', function (e) {
          if (e.target == this) {
            savefileto.types.forEach(type => {
              document.getElementById('savefileto-context-save' + type).hidden = true;
            });
          }
        });
    },
    
    onUnload: function()
    {
        if (this.accessKeyChangeListener)
            this.savefileto.prefs.removeListener(this.accessKeyChangeListener);
    },
    
    _updateMenuItemAccessKey: function(type)
    {
        var menu = document.getElementById("savefileto-context-save" + type);
        menu.accessKey = this.savefileto.prefs[type + "AccessKey"];
    }
}


/*
 * http://kb.mozillazine.org/Firefox_:_FAQs_:_About:config_Entries#Browser..2A
 * 
 * browser.download
 *  .useDownloadDir        bool  false: prompt; true: no prompt, download to folder defined in .folderList
 *  .folderList            int   0: desktop; 1: downloads folder; 2: folder in .dir
 *  .dir                   str   user defined download folder
 *  .lastDir               str   last folder used in Save Xxx As...
 *  .save_converter_index  int   the format to save web pages in (Save Page As...)
 *
 * .lastDir does not affect downloading but it is overwritten after it.
 * The original value is preserved because we don't want to affect the normal
 * behavior of Save Xxx As....
 * .save_converter_index on the other hand is always prompted when Save Page to
 * is used (skipPrompt has no effect) so there is no point in keeping the original value.
 *
 * See toolkit.jar:content/global/contentAreaUtils.js:getTargetFile() for actual implementation.
 *
 * Note that ff3's saveLink() doesn't honor the above preferences.
 * See savefileto_saveLink_prompt() below for details.
 */


// based on browser.jar:content/browser/browser.js:nsContextMenu.saveImage()  (context menu > Save Image as...)
function savefileto_save_image(folder, prompt)
{
    // ff3.1+ uses mediaURL
    var imageUrl = (gContextMenu.mediaURL ? gContextMenu.mediaURL : gContextMenu.imageURL);
    var contentDisposition = gContextMenu.contentData.contentDisposition;
    var contentType = gContextMenu.contentData.contentType;
    // ff3 has no docURL
    var docUri = (gContextMenu.docURL ? makeURI(gContextMenu.docURL) : gBrowser.currentURI);
    let referrerInfo = new ReferrerInfo(Ci.nsIHttpChannel.REFERRER_POLICY_UNSET, true, docUri);
    var principal = (gContextMenu.docURL ? gContextMenu.docURL : gBrowser.contentPrincipal);
    urlSecurityCheck(imageUrl, principal);
    let privacyContext = PrivateBrowsingUtils.privacyContextFromWindow(window);
    var prefsBackup = this.savefileto.getDownloadPrefs(docUri, privacyContext);
    var prefs = {useDownloadDir: (! prompt), folderList: 2, dir: folder.file, lastDir: folder.file};
    var lastDownloadDir = {uri: docUri, file: folder.file};
    var cookieJarSettings = gContextMenu.contentData.cookieJarSettings;
    this.savefileto.setDownloadPrefs({downloadPrefs: prefs, lastDownloadDir: lastDownloadDir}, privacyContext);
    // toolkit.jar:content/global/contentAreaUtils.js
    messageManager.addMessageListener('isTabPrivate', function chromeListenerImg (message) {
      messageManager.removeMessageListener('isTabPrivate', chromeListenerImg);
      internalSave(imageUrl, null, null, null, contentDisposition, contentType,
                   false, "SaveImageTitle", null, referrerInfo, cookieJarSettings,
                   null, (! prompt), null, message.data, principal);
      // #42: Since ff27 (?) if prompt is used, change-save-restore -hack doesn't work anymore.
      // I haven't looked why. Maybe related some ff async'fication change.
      // Workaround is the same as in overlay.js:savefileto_save_link_prompt(), don't restore original pref values.
      if (prompt) {
          prefsBackup.downloadPrefs.lastDir = prefs.lastDir;
          prefsBackup.lastDownloadDir = null;
      }
      
      // restore original browser download preferences
      savefileto.savefileto.setDownloadPrefs(prefsBackup, privacyContext);
    });
    gBrowser.selectedBrowser.messageManager.loadFrameScript('data:application/javascript;charset=UTF-8,' + encodeURIComponent('(' + (function () {
      sendAsyncMessage('isTabPrivate', docShell.QueryInterface(Components.interfaces.nsILoadContext).usePrivateBrowsing);
    }).toString() + ')();'), false);
}


/* 
 * It turns out that there's no "built-in" Content-Disposition filename support
 * in ff2. This is fixed in bug 299372 for ff3, but it only handles situations 
 * where filename is prompted, so no-prompt download must be made by hand. 
 */

function savefileto_save_link(folder, prompt)
{
    if (prompt)
        savefileto_save_link_prompt.call(this, folder);
    else
        savefileto_save_link_noPrompt.call(this, folder);
}


function savefileto_save_link_prompt(folder)
{
    var docUri = Services.io.newURI(gContextMenu.linkURL);
    let privacyContext = PrivateBrowsingUtils.privacyContextFromWindow(window);
    var prefsBackup = this.savefileto.getDownloadPrefs(docUri, privacyContext);
    try
    {
        var prompt = true;
        var prefs = {useDownloadDir: (! prompt), folderList: 2, dir: folder.file, lastDir: folder.file};
        var lastDownloadDir = {uri: docUri, file: folder.file};
        this.savefileto.setDownloadPrefs({downloadPrefs: prefs, lastDownloadDir: lastDownloadDir}, privacyContext);
        
        let referrerInfo = gContextMenu.onLink
          ? gContextMenu.contentData.linkReferrerInfo
          : gContextMenu.contentData.referrerInfo;
        let isContentWindowPrivate = gContextMenu.ownerDoc.isPrivate;
        let principal = gContextMenu.docURL ? gContextMenu.docURL : gBrowser.contentPrincipal;
        internalSave(
          gContextMenu.linkURL,
          null,
          null,
          gContextMenu.linkTextStr,
          null,
          'null',
          false,
          null,
          null,
          referrerInfo,
          null,
          gContextMenu.ownerDoc,
          false,
          null,
          isContentWindowPrivate,
          principal
        );
       // gContextMenu.saveLink não pode ser usado porque ele processa o link passando por redirecionamento, e em caso de redirecionamento para outro host o diálogo não vai abrir na pasta escolhida, pois ela abre na última pasta salva naquele host, o que o savefileto força ali em cima via setDownloadPrefs.
    }
    finally
    {
        // ff3 & saveLink():
        //  - doesn't honor |browser.download.*| prefs, it uses always |lastDir|
        //  - download dir is determined after saveLink() has returned, so our change-saveLink()-restore workaround doesn't work
        //
        // So the only option is NOT to restore |lastDir| to its original value.
        prefsBackup.downloadPrefs.lastDir = prefs.lastDir;
        
        // same thing with per-site download dir - don't restore original value
        prefsBackup.lastDownloadDir = null;
        
        // restore original browser download preferences
        this.savefileto.setDownloadPrefs(prefsBackup, privacyContext);
    }
}


function savefileto_save_link_noPrompt(folder)
{
    var linkUrl = gContextMenu.linkURL;
    var docUri = (gContextMenu.docURL ? makeURI(gContextMenu.docURL, gContextMenu.contentData.charSet) : gBrowser.currentURI);
    var principal = (gContextMenu.docURL ? gContextMenu.docURL : gBrowser.contentPrincipal);
    urlSecurityCheck(linkUrl, principal);
    var cookieJarSettings = gContextMenu.contentData.cookieJarSettings;
    
    // source url
    var sourceUri = gContextMenu.getLinkURI();
    sourceUri.QueryInterface(Ci.nsIURL);
    
    // target dir, file, uri
    var targetDir = folder.file;
    var targetFileName = getDefaultFileName(null, sourceUri, null, null);
    var targetFile = savefileto_getTargetFile(targetDir, targetFileName);
    var targetUri = Services.io.newFileURI(targetFile);
    
    messageManager.addMessageListener('isTabPrivate', function chromeListenerImg (message) {
      messageManager.removeMessageListener('isTabPrivate', chromeListenerImg);
      let isPrivate = message.data;
      
      // nsIWebProgressListener to listen possible content-disposition
      function WebProgressListener()
      {
          this.creationTime = Date.now();
      };
      
      WebProgressListener.prototype =
      {
          // nsISupports
          QueryInterface: function(iid)
          {
              if (iid.equals(Ci.nsIWebProgressListener) ||
                  iid.equals(Ci.nsIWebProgressListener2) ||
                  iid.equals(Ci.nsISupports))
                  return this;
              throw Cr.NS_ERROR_NO_INTERFACE;
          },
          
          // nsIWebProgressListener2
          onProgressChange64: function(webProgress, request, curSelfProgress, maxSelfProgress, curTotalProgress, maxTotalProgress) {},
          onRefreshAttempted: function(webProgress, refreshURI, millis, sameURI) {},
          
          // nsIWebProgressListener
          onLocationChange: function(webProgress, request, location) {},
          onProgressChange: function(webProgress, request, curSelfProgress, maxSelfProgress, curTotalProgress, maxTotalProgress) {},
          onSecurityChange: function(webProgress, request, state) {},
          onStatusChange: function(webProgress, request, status, message) {},
          
          onStateChange: function(webProgress, request, stateFlags, status)
          {
              this._onStateChange(webProgress, request, stateFlags, status);
              if (this.transfer)
                  this.transfer.onStateChange(webProgress, request, stateFlags, status);
          },
          
          _onStateChange: function(webProgress, request, stateFlags, status)
          {
              if (stateFlags & Ci.nsIWebProgressListener.STATE_START)
              {
                  if (this.transfer)
                      return;
                  
                  var trueTargetUri = targetUri;
                  
                  try
                  {
                      // change target file
                      // if request has been redirected, channel.URI != sourceUri (== channel.originalURI)
                      // or if there's content disposition header that which contains filename
                      var channel = request.QueryInterface(Ci.nsIHttpChannel);
                      var contentDisposition = null;
                      try { contentDisposition = channel.getResponseHeader("Content-Disposition"); } catch (e) { /* ignore */ }
                      var trueFilename = getDefaultFileName(null, channel.URI, null, contentDisposition);
                      if (trueFilename != targetFileName)
                      {
                          var trueTargetFile = savefileto_getTargetFile(targetDir, trueFilename);
                          trueTargetUri = Services.io.newFileURI(trueTargetFile);
                          this.trueTargetFilename = trueTargetFile.leafName;
                          this.tempFile = targetFile;
                      }
                  }
                  catch (e) { if (e.result == Cr.NS_ERROR_NO_INTERFACE) { /* ignore */ } else { throw Error(e); } }
                  
                  this.initTransfer(trueTargetUri, this.tempFile);
              }
              else if (stateFlags & Ci.nsIWebProgressListener.STATE_STOP)
              {
                  if (! this.tempFile)
                      return;
                  
                  // rename temp file to true target file
                  this.tempFile.moveTo(null, this.trueTargetFilename);
                  delete this.tempFile;
              }
          },
          
          initTransfer: function(trueTargetUri, tempFile)
          {
              if (this.transfer)
                  return;
              
              // ff1.5 seems to leak if notify isn't removed
              timer.cancel();
              delete callback.notify;
              
              var transfer = this.transfer = Cc["@mozilla.org/transfer;1"].createInstance(Ci.nsITransfer);
              var displayName = null;
              var mimeInfo = null;
              var time = this.creationTime * 1000;
              transfer.init(sourceUri, null, trueTargetUri, displayName, mimeInfo, time, tempFile, persist, isPrivate, Ci.nsITransfer.DOWNLOAD_ACCEPTABLE, null);
              
              // nsIWebProgressListener2.onRefreshAttempted is ff3 only
              for (var _name of ["onLocationChange", "onProgressChange", "onSecurityChange", "onStatusChange", "onProgressChange64", "onRefreshAttempted"])
              {
                  let name = _name;
                  if (transfer[name])
                      this[name] = function() { transfer[name].apply(transfer, arguments); };
              }
          }
      }
      
      // nsIWebBrowserPersist
      var persist = Cc["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Ci.nsIWebBrowserPersist);
      var progressListener = new WebProgressListener(persist);
      persist.progressListener = progressListener;
      persist.persistFlags =
          Ci.nsIWebBrowserPersist.PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION |
          Ci.nsIWebBrowserPersist.PERSIST_FLAGS_BYPASS_CACHE |
          Ci.nsIWebBrowserPersist.PERSIST_FLAGS_FORCE_ALLOW_COOKIES |
          Ci.nsIWebBrowserPersist.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
      
      // timeout for waiting the HTTP headers
      // otherwise it would be impossible to cancel the download from the ui
      var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
      var callback = {notify: function() { progressListener.initTransfer(targetUri, null); }};
      var timeout = savefileto.savefileto.prefs.responseTimeout;
      timer.initWithCallback(callback, timeout, Ci.nsITimer.TYPE_ONE_SHOT);
      
      var refererInfo = new ReferrerInfo(Ci.nsIHttpChannel.REFERRER_POLICY_UNSET, true, gBrowser.currentURI);
      persist.savePrivacyAwareURI(sourceUri, principal, null, refererInfo, cookieJarSettings, null, null, targetUri, Ci.nsIContentPolicy.TYPE_SAVEAS_DOWNLOAD, isPrivate);
    });
    gBrowser.selectedBrowser.messageManager.loadFrameScript('data:application/javascript;charset=UTF-8,' + encodeURIComponent('(' + (function () {
      sendAsyncMessage('isTabPrivate', docShell.QueryInterface(Components.interfaces.nsILoadContext).usePrivateBrowsing);
    }).toString() + ')();'), false);
}


function savefileto_save_page(folder, prompt)
{
    var docUri = gBrowser.currentURI;
    let privacyContext = PrivateBrowsingUtils.privacyContextFromWindow(window);
    var prefsBackup = this.savefileto.getDownloadPrefs(docUri, privacyContext);
    try
    {
        var prefs = {useDownloadDir: (! prompt), folderList: 2, dir: folder.file, lastDir: folder.file};
        var lastDownloadDir = {uri: docUri, file: folder.file};
        this.savefileto.setDownloadPrefs({downloadPrefs: prefs, lastDownloadDir: lastDownloadDir}, privacyContext);
        // toolkit.jar:content/global/contentAreaUtils.js
        // saveDocument(aDocument, aSkipPrompt)
        // aSkipPrompt has no effect - prompts always
        saveBrowser(gBrowser.selectedBrowser, (! prompt));
    }
    finally
    {
        // #42: Since ff27 (?) if prompt is used, change-save-restore -hack doesn't work anymore.
        // I haven't looked why. Maybe related some ff async'fication change.
        // Workaround is the same as in overlay.js:savefileto_save_link_prompt(), don't restore original pref values.
        if (prompt) {
            prefsBackup.downloadPrefs.lastDir = prefs.lastDir;
            prefsBackup.lastDownloadDir = null;
        }
        
        // restore original browser download preferences
        this.savefileto.setDownloadPrefs(prefsBackup, privacyContext);
    }
}


function savefileto_getTargetFile(dir, filename)
{
    var file = dir.clone();
    file.append(filename);
    
    // copy paste from toolkit.jar:content/global/contentAreaUtils.js:getTargetFile()
    var collisionCount = 0;
    while (file.exists()) {
        collisionCount++;
        if (collisionCount == 1) {
            // Append "(2)" before the last dot in (or at the end of) the filename
            // special case .ext.gz etc files so we don't wind up with .tar(2).gz
            if (file.leafName.match(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i))
                file.leafName = file.leafName.replace(/\.[^\.]{1,3}\.(gz|bz2|Z)$/i, "(2)$&");
            else
                file.leafName = file.leafName.replace(/(\.[^\.]*)?$/, "(2)$&");
        }
        else {
            // replace the last (n) in the filename with (n+1)
            file.leafName = file.leafName.replace(/^(.*\()\d+\)/, "$1" + (collisionCount+1) + ")");
        }
    }
    
    return file;
}


window.addEventListener("load", function() { savefileto.onLoad(); }, false);
window.addEventListener("unload", function() { savefileto.onUnload(); }, false);
