/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

var savefileto =
{
    // const values from chrome://browser/content/pageinfo/pageInfo.js are no longer visible
    COL_IMAGE_ADDRESS: 0,
    COL_IMAGE_TYPE:    1,
    COL_IMAGE_SIZE:    2,
    COL_IMAGE_ALT:     3,
    COL_IMAGE_COUNT:   4,
    COL_IMAGE_NODE:    5,
    COL_IMAGE_BG:      6,
    
    onLoad: function()
    {
        Components.utils.import("chrome://savefileto-modules/content/savefileto.jsm", this);
        
        if (! this.savefileto.prefs.imageEnabled)
            return;
        
        // save single image
        this.mediaSaveAsButton1 = document.getElementById("imagesaveasbutton");
        this.saveFileToButton1 = this.createButton(this.mediaSaveAsButton1);
        this.deck1 = this.saveFileToButton1.parentNode;
        
        // save multiple images
        this.mediaSaveAsButton2 = document.getElementById("mediasaveasbutton");
        this.saveFileToButton2 = this.createButton(this.mediaSaveAsButton2);
        this.deck2 = this.saveFileToButton2.parentNode;
        
        let self = this;
        let commandCallback = function(e, folder) { e.stopPropagation(); self.onCommand(e, folder); };
        let options = {mainMenuPosition: "after_start"};
        let cascadeMenu = this.savefileto.createCascadeMenu(window, "image", commandCallback, options);
        cascadeMenu.attach(this.saveFileToButton1);
        cascadeMenu.attach(this.saveFileToButton2);
        
        let tree = document.getElementById("imagetree");
        tree.addEventListener("select", function() { self.onMediaTreeSelect(); }, false);
    },
    
    createButton: function(oldButton)
    {
        /**
         * <button (old) />
         *  =>
         * <deck class="savefileto">
         *   <hbox>
         *     <button (old) />
         *     <spacer flex="1" />
         *   </hbox>
         *   <button (new) />
         * </deck>
         */
        document.getElementById('imageSaveBox').removeAttribute('align');
        let newButton = document.createXULElement('button');
        newButton.id = 'savefiletobutton';
        newButton.label = 'Save File To';
        newButton.type = 'menu';
        newButton.accessKey = 'T';
        oldButton.insertAdjacentElement('afterend', newButton);
        
        return newButton;
    },
    
    onMediaTreeSelect: function()
    {
        let tree = document.getElementById("imagetree");
        let rows = getSelectedRows(tree);
        let data = rows.map(row => gImageView.data[row]);
        let enabled = data.every(d => (d[this.COL_IMAGE_NODE].HTMLImageElement || d[this.COL_IMAGE_BG]));
        
        this.deck1.selectedIndex = (enabled ? 1 : 0);
        this.deck2.selectedIndex = (enabled ? 1 : 0);
    },
    
    onCommand: function(e, folder)
    {
        let dir = folder.file;
        let prompt = (e.ctrlKey || this.savefileto.prefs.prompt);
        
        let tree = document.getElementById("imagetree");
        let rows = getSelectedRows(tree);
        let data = rows.map(row => gImageView.data[row]);
        let urls = data.map(d => ({imageUrl: d[this.COL_IMAGE_ADDRESS], docUri: makeURI(d[this.COL_IMAGE_NODE].baseURI)}));
        
        let self = this;
        let postSaveCallback = function() { self.savefileto.addToRecent("image", folder, window); };
        
        if (urls.length == 1)
            this.saveImage(urls[0].imageUrl, urls[0].docUri, dir, prompt, postSaveCallback);
        else
            this.saveImages(urls, dir, prompt, postSaveCallback);
    },
    
    saveImage: function(imageUrl, docUri, dir, prompt, postSaveCallback)
    {
        this._saveImage(imageUrl, docUri, dir, prompt);
        postSaveCallback();
    },
    
    saveImages: function(urls, dir, prompt, postSaveCallback)
    {
        let self = this;
        
        let saveImagesToFolder = function(dir)
        {
            if (! dir)
                return;
            
            for (let {imageUrl, docUri} of Object.values(urls))
                self._saveImage(imageUrl, docUri, dir, false);
            
            postSaveCallback();
        }
        
        if (! prompt)
            saveImagesToFolder(dir);
        else
        {
            this.savefileto.preserveDownloadPrefs(null, this, function()
            {
                // selectSaveFolder() defaults to folder set in download prefs - override it
                let prefs = {useDownloadDir: (! prompt), folderList: 2, dir: dir, lastDir: dir};
                this.savefileto.setDownloadPrefs({downloadPrefs: prefs});
                
                selectSaveFolder(saveImagesToFolder);
            });
        }
    },
    
    _saveImage: function(imageUrl, docUri, dir, prompt)
    {
        this.savefileto.preserveDownloadPrefs(docUri, this, function()
        {
            let prefs = {useDownloadDir: (! prompt), folderList: 2, dir: dir, lastDir: dir};
            let lastDownloadDir = {uri: docUri, file: dir};
            this.savefileto.setDownloadPrefs({downloadPrefs: prefs, lastDownloadDir: lastDownloadDir});
            
            // chrome://global/content/contentAreaUtils.js
            // saveURL(
            //     aURL, aFileName, aFilePickerTitleKey, aShouldBypassCache,
            //     aSkipPrompt, aReferrer, aSourceDocument, aIsContentWindowPrivate)
            let referrerInfo = new ReferrerInfo(Ci.nsIHttpChannel.REFERRER_POLICY_UNSET, true, docUri);
            saveURL(
                imageUrl, null, null, "SaveImageTitle", false, 
                (! prompt), referrerInfo, E10SUtils.deserializeCookieJarSettings(gDocInfo.cookieJarSettings), null, gDocInfo.isContentWindowPrivate, gDocInfo.principal);
        });
    }
}


window.addEventListener("load", function() { savefileto.onLoad(); }, false);
