(function () {
  function pref(name, value) {
    let branch = Services.prefs.getBranch("");
    let defaultBranch = Services.prefs.getDefaultBranch("");
    if (defaultBranch.getPrefType(name) == Components.interfaces.nsIPrefBranch.PREF_INVALID) {
      // Only use the default branch if it doesn't already have the pref set.
      // If there is already a pref with this value on the default branch, the
      // extension wants to override a built-in value.
      branch = defaultBranch;
    } else if (defaultBranch.prefHasUserValue(name)) {
      // If a pref already has a user-set value it proper type
      // will be returned (not PREF_INVALID). In that case keep the user's
      // value and overwrite the default.
      branch = defaultBranch;
    }

    if (typeof value == "boolean") {
      branch.setBoolPref(name, value);
    } else if (typeof value == "string") {
      if (value.startsWith("chrome://") && value.endsWith(".properties")) {
        let valueLocal = Components.classes["@mozilla.org/pref-localizedstring;1"]
                         .createInstance(Components.interfaces.nsIPrefLocalizedString);
        valueLocal.data = value;
        branch.setComplexValue(name, Components.interfaces.nsIPrefLocalizedString, valueLocal);
      } else {
        branch.setStringPref(name, value);
      }
    } else if (typeof value == "number" && Number.isInteger(value)) {
      branch.setIntPref(name, value);
    } else if (typeof value == "number" && Number.isFloat(value)) {
      // Floats are set as char prefs, then retrieved using getFloatPref
      branch.setCharPref(name, value);
    }
  }

  // *** see platform/<platform>/defaults/preferences for platform specific preferences ***

  // localized extension description
  pref("extensions.savefileto@mozdev.org.description", "chrome://savefileto/locale/savefileto.properties");

  // false: use subMenuPosition value; true: each child menu level is opened on alternating sides
  pref("extensions.savefileto@mozdev.org.alternatingMenuPosition", false);
  // clear recent folders at browser exit
  pref("extensions.savefileto@mozdev.org.clearRecent", false);
  // confirm removes in prefwindow
  pref("extensions.savefileto@mozdev.org.confirmRemove", true);
  // file name filter, '|' separated globs
  pref("extensions.savefileto@mozdev.org.filter", "CVS");
  // popup position for main menu - https://developer.mozilla.org/en/XUL/PopupGuide/Positioning
  pref("extensions.savefileto@mozdev.org.mainMenuPosition", "end_before");
  // maximum number of menu items (0 = unlimited)
  pref("extensions.savefileto@mozdev.org.maxMenuLength", 0);
  // minimum number of menu items
  pref("extensions.savefileto@mozdev.org.minMenuLength", 5);
  // use save dialog
  pref("extensions.savefileto@mozdev.org.prompt", false);
  // order recent folders: lastUsed, name
  pref("extensions.savefileto@mozdev.org.recentOrder", "lastUsed");
  // number of recent folders to be shown
  pref("extensions.savefileto@mozdev.org.recentSize", 5);
  // time to wait for reponse HTTP headers (milliseconds)
  pref("extensions.savefileto@mozdev.org.responseTimeout", 3000);
  // start download as soon as folder is selected in save file dialog
  pref("extensions.savefileto@mozdev.org.startDownloadOnSelect", true);
  // popup position for submenus - https://developer.mozilla.org/en/XUL/PopupGuide/Positioning
  pref("extensions.savefileto@mozdev.org.subMenuPosition", "end_before");
  // don't make sub-menus for empty folders
  pref("extensions.savefileto@mozdev.org.trimEmptyMenu", false);

  // *** PAGE ***

  // is menu item shown in context menu
  pref("extensions.savefileto@mozdev.org.page.enabled", true);
  // menu item's access key in context menu
  pref("extensions.savefileto@mozdev.org.page.accessKey", "chrome://savefileto/locale/savefileto.properties");
  // menu entries json
  pref("extensions.savefileto@mozdev.org.page.entries", '[{"type":"recent"},{"type":"separator"},{"type":"named","value":"Desk","menu":true},{"type":"named","value":"Home","menu":true},{"type":"separator"},{"type":"item","command":"cmd_savefileto_openPrefs_page"}]');
  // recent folders json
  pref("extensions.savefileto@mozdev.org.page.recent", '[]');

  // *** LINK ***

  // is menu item shown in context menu
  pref("extensions.savefileto@mozdev.org.link.enabled", true);
  // menu item's access key in context menu
  pref("extensions.savefileto@mozdev.org.link.accessKey", "chrome://savefileto/locale/savefileto.properties");
  // menu entries json
  pref("extensions.savefileto@mozdev.org.link.entries", '[{"type":"recent"},{"type":"separator"},{"type":"named","value":"Desk","menu":true},{"type":"named","value":"Home","menu":true},{"type":"separator"},{"type":"item","command":"cmd_savefileto_openPrefs_link"}]');
  // recent folders json
  pref("extensions.savefileto@mozdev.org.link.recent", '[]');

  // *** IMAGE ***

  // is menu item shown in context menu
  pref("extensions.savefileto@mozdev.org.image.enabled", true);
  // menu item's access key in context menu
  pref("extensions.savefileto@mozdev.org.image.accessKey", "chrome://savefileto/locale/savefileto.properties");
  // menu entries json
  pref("extensions.savefileto@mozdev.org.image.entries", '[{"type":"recent"},{"type":"separator"},{"type":"named","value":"Desk","menu":true},{"type":"named","value":"Home","menu":true},{"type":"separator"},{"type":"item","command":"cmd_savefileto_openPrefs_image"}]');
  // recent folders json
  pref("extensions.savefileto@mozdev.org.image.recent", '[]');

  // *** FILE ***

  // is shown in Save File dialog (unknownContentType.xul)
  pref("extensions.savefileto@mozdev.org.file.enabled", true);
  // menu item's access key in dialog
  pref("extensions.savefileto@mozdev.org.file.accessKey", "chrome://savefileto/locale/savefileto.properties");
  // menu entries json
  pref("extensions.savefileto@mozdev.org.file.entries", '[{"type":"recent"},{"type":"separator"},{"type":"named","value":"Desk","menu":true},{"type":"named","value":"Home","menu":true},{"type":"separator"},{"type":"item","command":"cmd_savefileto_openPrefs_file"}]');
  // recent folders json
  pref("extensions.savefileto@mozdev.org.file.recent", '[]'); 
})()