/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

const { DownloadPaths } = ChromeUtils.importESModule('resource://gre/modules/DownloadPaths.sys.mjs');
const { PrivateBrowsingUtils } = ChromeUtils.importESModule('resource://gre/modules/PrivateBrowsingUtils.sys.mjs');

var savefileto =
{
    get mode()   { return document.getElementById("mode"); },
    get radio()  { return document.getElementById("savefileto-radio"); },
    get button() { return document.getElementById("savefileto-button"); },
    get save()   { return document.getElementById("savefileto-save-button"); },
    
    get selected()      { return this.radio.selected; },
    set selected(value) { this.mode.selectedItem = this.radio; },
    
    get folder() { return this._folder; },
    set folder(value)
    {
        this._folder = value;
        this.save.disabled = (! value);
    },
    
    onLoad: function()
    {
        Components.utils.import("chrome://savefileto-modules/content/common.jsm", this);
        Components.utils.import("chrome://savefileto-modules/content/DataSource.jsm", this);
        Components.utils.import("chrome://savefileto-modules/content/savefileto.jsm", this);
        
        var self = this;
        
        var enabled = this.savefileto.prefs.fileEnabled;
        document.getElementById("savefileto-box").hidden = (! enabled);
        
        if (! enabled)
            return;
        
        // show normalBox instead of basicBox
        // idea copied from DownThemAll!
        // implementation is slightly different, so the actual result varies depending whose load handler is run first
        if (! document.getElementById("basicBox").collapsed)
        {
            document.getElementById("open").parentNode.collapsed = true;
            document.getElementById("basicBox").collapsed = true;
            document.getElementById("normalBox").collapsed = false;
        }
        
        this.prompt = false;
        
        // init access key
        this._updateAccessKey();
        this.accessKeyChangeListener = function(prefs, propName) { if (propName == "fileAccessKey") self._updateAccessKey(); }
        this.savefileto.prefs.addListener(this.accessKeyChangeListener);
        
        // default to most recent folder if there is one
        this.folder = this.savefileto.getMostRecentFolder("file", window);
        this.button.label = (this.folder ? this.folder.label : "");
        
        // create cascade menu
        var commandCallback = function(e, folder) { self.onCommand(e, folder); }
        let options = {mainMenuPosition: "after_start"};
        let cascadeMenu = this.savefileto.createCascadeMenu(window, "file", commandCallback, options);
        cascadeMenu.attach(this.button);
        
        // select Save File to when button popup opened
        var menuPopup = this.button.getElementsByTagName('menupopup')[0];
        menuPopup.addEventListener("popupshowing", function() { self.selected = true; }, false);
        
        // pre and post mode select listeners
        document.addEventListener("select", function(e) { self.onBeforeModeSelect(e); }, true);
        //this.mode.addEventListener("select", function(e) { dump("mode select\n"); }, false);
        this.radio.addEventListener("RadioStateChange", function(e) { self.onAfterModeSelect(e); }, false);
        this.oldSelected = false;
        
        // dialog accept listener
        dialog.onOK = (function () {
            var cached_function = dialog.onOK;

            return function () {
              if (savefileto.selected)
                return savefileto.onDialogAccept();
              else
                return cached_function.apply(this, arguments);
            };
          }
        )();
    },
    
    onUnload: function()
    {
        if (this.accessKeyChangeListener)
            this.savefileto.prefs.removeListener(this.accessKeyChangeListener);
    },
    
    onBeforeModeSelect: function(e)
    {
        if (e.target != this.mode)
            return;
        
        var selected = this.selected;
        if (selected || (selected == this.oldSelected))
            return;
        
        this._update();
    },
    
    onAfterModeSelect: function(e)
    {
        var selected = this.selected;
        if ((! selected) && (selected == this.oldSelected))
            return;
        
        this._update();
    },
    
    onCommand: function(e, folder)
    {
        this.folder = folder;
        this.prompt = e.ctrlKey;
        this.button.label = this.folder.pathLabel;
        
        this._update();
        
        if (this.savefileto.prefs.startDownloadOnSelect)
            document.getElementById('unknownContentType').acceptDialog();
    },
    
    onSaveCommand: function(e)
    {
        this.selected = true;
        this.prompt = e.ctrlKey;
        
        this._update();
        document.getElementById('unknownContentType').acceptDialog();
    },
    
    // http://mxr.mozilla.org/firefox/source/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in#850
    // http://mxr.mozilla.org/firefox/source/toolkit/mozapps/downloads/src/nsHelperAppDlg.js.in#506
    
    onDialogAccept: function()
    {
        var self = this;
        var launcher = dialog.mLauncher;
        var prompt = (this.prompt || this.savefileto.prefs.prompt);
        var folder = this.folder;
        var file = null;
        let context = dialog.mContext;
        
        if (! prompt)
        {
            //https://searchfox.org/mozilla-central/rev/d85572c1963f72e8bef2787d900e0a8ffd8e6728/toolkit/mozapps/downloads/HelperAppDlg.jsm#482-501
            let url = dialog.mLauncher.source;

            if (url instanceof Ci.nsINestedURI) {
            url = url.innermostURI;
            }

            let iconPath = 'goat';
            let fname = '';
            if (dialog.mLauncher.suggestedFileName) {
                fname = iconPath = dialog.mLauncher.suggestedFileName;
            } else if (url instanceof Ci.nsIURL) {
                // A url, use file name from it.
                fname = iconPath = url.fileName;
            } else if (['data', 'blob'].includes(url.scheme)) {
                // The path is useless for these, so use a reasonable default.
                let { MIMEType } = dialog.mLauncher.MIMEInfo;
                fname = Cc['@mozilla.org/mime;1'].getService(Ci.nsIMIMEService).getValidFileName(null, MIMEType, url, 0);
            } else {
                fname = url.pathQueryRef;
            }

            file = folder.file.clone();
            file.append(fname);
            file = DownloadPaths.createNiceUniqueFile(file);
        }
        
        // clean up (from nsHelperAppDlg.js.in)
        launcher.setWebProgressListener(null);
        dialog.mDialog.dialog = null;
        
        // don't start downloading until dialog has closed
        var _promptForSaveDestination = function() { self.callLater(0, self, self._promptForSaveDestination, [launcher, prompt, folder, file, context]); };
        window.addEventListener("unload", _promptForSaveDestination, false);
        
        return true;
    },
    
    _promptForSaveDestination: function(launcher, prompt, folder, file, context)
    {
        var docUri = launcher.source;
        var privacyContext = PrivateBrowsingUtils.privacyContextFromWindow(context);
        var prefsBackup = this.savefileto.getDownloadPrefs(docUri, privacyContext);

        try
        {
            var prefs = {useDownloadDir: (! prompt), folderList: 2, dir: folder.file, lastDir: folder.file};
            var lastDownloadDir = {uri: docUri, file: folder.file};
            this.savefileto.setDownloadPrefs({downloadPrefs: prefs, lastDownloadDir: lastDownloadDir}, privacyContext);

            // promptForSaveDestination() throws if its prompt dialog is cancelled - don't bother catching
            if (prompt) {
              launcher.promptForSaveDestination();
            } else {
              launcher.MIMEInfo.preferredAction = launcher.MIMEInfo.saveToDisk;
              launcher.saveDestinationAvailable(file);
            }
            this.savefileto.addToRecent("file", folder, context);
        }
        finally
        {
            // #42: Since ff27 (?) if prompt is used, change-save-restore -hack doesn't work anymore.
            // I haven't looked why. Maybe related some ff async'fication change.
            // Workaround is the same as in overlay.js:savefileto_save_link_prompt(), don't restore original pref values.
            if (prompt)
            {
                prefsBackup.downloadPrefs.lastDir = prefs.lastDir;
                prefsBackup.lastDownloadDir = null;
            }
            
            // restore original browser download preferences
            this.savefileto.setDownloadPrefs(prefsBackup, privacyContext);
        }
    },
    
    _updateAccessKey: function()
    {
        this.radio.accessKey = this.savefileto.prefs.fileAccessKey;
    },
    
    _update: function()
    {
        var selected = this.selected;
        var hasFolder = this.folder;
        
        this.oldSelected = selected;
        
        this.button.style.MozUserFocus = (selected ? "normal" : "ignore");
        this.save.style.MozUserFocus = (selected ? "normal" : "ignore");
        document.getElementById("openHandler").style.MozUserFocus = (selected ? "ignore" : "normal");
        document.getElementById("chooseButton").style.MozUserFocus = (selected ? "ignore" : "normal");
        document.getElementById("rememberChoice").disabled = selected;
        document.getElementById('unknownContentType').getButton("accept").disabled = (selected && (! hasFolder));
    }
}


window.addEventListener("load", function() { savefileto.onLoad(); }, false);
window.addEventListener("unload", function() { savefileto.onUnload(); }, false);
