/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */


var EXPORTED_SYMBOLS =
[
    "savefileto"
];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;
const Cr = Components.results;

const { PrivateBrowsingUtils } = ChromeUtils.importESModule('resource://gre/modules/PrivateBrowsingUtils.sys.mjs');
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

Cu.import("chrome://savefileto-modules/content/CascadeMenu.jsm");
Cu.import("chrome://savefileto-modules/content/common.jsm");
Cu.import("chrome://savefileto-modules/content/constants.jsm");
Cu.import("chrome://savefileto-modules/content/DataSource.jsm");
Cu.import("chrome://savefileto-modules/content/Entry.jsm");
Cu.import("chrome://savefileto-modules/content/Preferences.jsm");
Cu.import("chrome://savefileto-modules/content/Transformer.jsm");


var savefileto =
{
    get prefs()
    {
        var prefNames =
        [
            "page.enabled",
            "link.enabled",
            "image.enabled",
            "file.enabled",
            
            "page.accessKey",
            "link.accessKey",
            "image.accessKey",
            "file.accessKey",
            
            "alternatingMenuPosition",
            "mainMenuPosition",
            "maxMenuLength",
            "minMenuLength",
            "prompt",
            "recentSize",
            "responseTimeout",
            "startDownloadOnSelect",
            "subMenuPosition",
            "trimEmptyMenu"
        ];
        
        var prefs = new PreferencesProxy(EXT_PREF_BRANCH, prefNames);
        return lazyInit(this, "prefs", prefs);
    },
    
    get bundle()
    {
        var url = "chrome://savefileto/locale/savefileto.properties";
        return lazyInit(this, "bundle", Services.strings.createBundle(url));
    },
    
    get folderNameBundle()
    {
        var url = "chrome://savefileto/locale/folderNames.properties";
        return lazyInit(this, "folderNameBundle", Services.strings.createBundle(url));
    },
    
    get commandLabelBundle()
    {
        var url = "chrome://savefileto/locale/commandLabels.properties";
        return lazyInit(this, "commandLabelBundle", new StringBundle(url));
    },
    
    get recentOrder()
    {
        let recentOrder = new PreferenceProxy(EXT_PREF_BRANCH + "recentOrder");
        return lazyInit(this, "recentOrder", recentOrder);
    },
    
    get folderFilter()
    {
        var glob = new PreferenceProxy(EXT_PREF_BRANCH + "filter");
        var fileNameFilter = new FileNameFilter(glob);
        var filter = function(file) { return fileNameFilter.filter(file.leafName); }
        return lazyInit(this, "folderFilter", filter);
    },
    
    get folderVariables()
    {
        // "Home,Desk,..." -> [{name: "$Home", value: <file>}, {name: "$Desk", value: <file>}, ...]
        let folderVariables = Services.prefs.getCharPref(EXT_PREF_BRANCH + "folderVariables");
        folderVariables = folderVariables.split(",").map(s => s.trim()).filter(s => s);
        folderVariables = folderVariables.map(s => ({name: s, value: getFile("$" + s)}));
        folderVariables = folderVariables.filter(f => (f.value && f.value.exists() && f.value.isDirectory()));
        return folderVariables;
    },
    
    get folderEntryTransformer()
    {
        var transformer = new FolderEntryTransformer(compare, this.folderFilter);
        return lazyInit(this, "folderEntryTransformer", transformer);
    },
    
    get recentFolderTransformer()
    {
        var transformer = new RecentFolderTransformer();
        return lazyInit(this, "recentFolderTransformer", transformer);
    },
    
    get namedFolderTransformer()
    {
        var transformer = new NamedFolderTransformer(this.folderNameBundle);
        return lazyInit(this, "namedFolderTransformer", transformer);
    },
    
    get namedFolderEntryTransformer()
    {
        var transformer = new ChainedTransformer(this.namedFolderTransformer, this.folderEntryTransformer);
        return lazyInit(this, "namedFolderEntryTransformer", transformer);
    },
    
    get commandLabelTransformer()
    {
        var transformer = new CommandLabelTransformer(this.commandLabelBundle);
        return lazyInit(this, "commandLabelTransformer", transformer);
    },
    
    mainDataSource:
    {
        get page()  { return lazyInit(this, "page",  savefileto._createMainDataSource("page")); },
        get link()  { return lazyInit(this, "link",  savefileto._createMainDataSource("link")); },
        get image() { return lazyInit(this, "image", savefileto._createMainDataSource("image")); },
        get file()  { return lazyInit(this, "file",  savefileto._createMainDataSource("file")); }
    },
    
    recentDataSource:
    {
        get page()  { return lazyInit(this, "page",  savefileto._createRecentDataSource("page")); },
        get link()  { return lazyInit(this, "link",  savefileto._createRecentDataSource("link")); },
        get image() { return lazyInit(this, "image", savefileto._createRecentDataSource("image")); },
        get file()  { return lazyInit(this, "file",  savefileto._createRecentDataSource("file")); }
    },
    
    _recentEntryDataSource: {},
    _menuEntryDataSource: {},
    
    createCascadeMenu: function(win, type, commandCallback, options)
    {
        var prefs = this.prefs;
        
        var config =
        {
            get alternatingMenuPosition() { return prefs.alternatingMenuPosition; },
            commandCallback: commandCallback,
            commandLabel: this.bundle.GetStringFromName("saveHere"),
            get dataSource() { return lazyInit(this, "dataSource", savefileto.getMenuEntryDataSource(type, win)); },
            get fallbackDataSource() { return lazyInit(this, "fallbackDataSource", {entries: [savefileto.getOpenPrefsCommandEntry(type)]}); },
            mainMenuClass: "savefileto-mainMenu",
            get mainMenuPosition() { return prefs.mainMenuPosition; },
            get maxMenuLength() { return prefs.maxMenuLength; },
            get minMenuLength() { return prefs.minMenuLength; },
            subMenuClass: "savefileto-subMenu",
            get subMenuPosition() { return prefs.subMenuPosition; },
            get trimEmptyMenu() { return prefs.trimEmptyMenu; }
        }
        
        if (options)
        {
            for (let name in options)
            {
                delete config[name];
                config[name] = options[name];
            }
        }
        
        return new CascadeMenu(config);
    },
    
    getSaveAsCommandEntry: function sft_getSaveAsCommandEntry(type)
    {
        var entry = {type: "item", command: "cmd_savefileto_saveAs_" + type};
        var result = this.commandLabelTransformer.transform(entry);
        return result;
    },
    
    getOpenPrefsCommandEntry: function sft_getOpenPrefsCommandEntry(type)
    {
        var entry = {type: "item", command: "cmd_savefileto_openPrefs_" + type};
        var result = this.commandLabelTransformer.transform(entry);
        return result;
    },
    
    addToRecent: function(type, entry, win)
    {
        var dataSource = this.getRecentEntryDataSource(type, win);
        var maxSize = this.prefs.recentSize;
        var equals = (a, b) => ((a.pathLabel == b.pathLabel) && (a.path == b.path));
        
        this._addToRecent(entry, dataSource, maxSize, equals);
    },
    
    _addToRecent: function sft__addToRecent(entry, dataSource, maxSize, equals)
    {
        var entries = dataSource.entries;
        var length = entries.length;
        var maxIndex = Math.min(length, maxSize - 1) - 1;
        
        for (var index = 0; index <= maxIndex; index++)
            if (equals(entry, entries[index]))
                break;
        
        // if the entry is the first already, don't change ds
        // note that ds may be over maxSize at this point, but that's ok because view trims the ds
        if ((index == 0) && (index <= maxIndex))
            return;
        
        entries = entries.slice();
        
        if (index <= maxIndex)
            entries.splice(index, 1);
        
        entries.unshift(entry);
        
        if (entries.length > maxSize)
            entries = entries.slice(0, maxSize);
        
        dataSource.entries = entries;
    },
    
    _createMainDataSource: function sft__createMainDataSource(type)
    {
        var prefName = EXT_PREF_BRANCH + type + ".entries";
        var dataSource = new PreferenceDataSource(prefName);
        return dataSource;
    },
    
    _createRecentDataSource: function(type)
    {
        var prefName = EXT_PREF_BRANCH + type + ".recent";
        var dataSource = new PreferenceDataSource(prefName);
        return dataSource;
    },
    
    _createRecentEntryDataSource: function(type, win)
    {
        // entry list transformers
        var getTransformer = new ArrayTransformer(this.folderEntryTransformer, true);
        var setTransformer = new ArrayTransformer(this.recentFolderTransformer, true);
        
        // limit recent list size
        var maxLength = {get value() { return savefileto.prefs.recentSize; }};
        getTransformer =
            new ChainedTransformer(
                getTransformer,
                new MaxLengthTransformer(maxLength));
        
        var dataSource =
            (PrivateBrowsingUtils.isWindowPrivate(win)
                ? new MemoryDataSource(this.recentDataSource[type].entries)
                : this.recentDataSource[type]);
        
        dataSource =
            new TransformingDataSource(
                dataSource,
                getTransformer,
                setTransformer);
        
        return dataSource;
    },
    
    _createMenuEntryDataSource: function(type, win)
    {
        var mainDataSource = this.mainDataSource[type];
        var recentEntryDataSource = this.getRecentEntryDataSource(type, win);
        
        // order recent folders
        recentEntryDataSource =
            new SortedRecentEntryDataSource(recentEntryDataSource, this.recentOrder);
        
        // transform data source entry list into cascade menu entry list
        var transformers =
        {
            folder: this.folderEntryTransformer,
            item: this.commandLabelTransformer,
            named: this.namedFolderEntryTransformer,
            recent: new DataSourceTransformer(recentEntryDataSource),
            separator: null
        }
        var transformer =
            new ArrayTransformer(
                new SwitchTransformer("type", transformers),
                true);
        
        var cachedDataSource =
            new CachingDataSource(
                new TransformingDataSource(mainDataSource, transformer));
        
        // reset cache when recent ds changes
        // (CachingDataSource handles internally changes in decorated ds (mainDs))
        recentEntryDataSource.addListener(function() { cachedDataSource.reset(); });
        
        return cachedDataSource;
    },
    
    getRecentEntryDataSource: function(type, win)
    {
        let isPrivate = PrivateBrowsingUtils.isWindowPrivate(win)
        
        if (! (isPrivate in this._recentEntryDataSource))
            this._recentEntryDataSource[isPrivate] = {};
        
        if (! (type in this._recentEntryDataSource[isPrivate]))
            this._recentEntryDataSource[isPrivate][type] = this._createRecentEntryDataSource(type, win);
            
        return this._recentEntryDataSource[isPrivate][type];
    },
    
    getMenuEntryDataSource: function(type, win)
    {
        let isPrivate = PrivateBrowsingUtils.isWindowPrivate(win)
        
        if (! (isPrivate in this._menuEntryDataSource))
            this._menuEntryDataSource[isPrivate] = {};
        
        if (! (type in this._menuEntryDataSource[isPrivate]))
            this._menuEntryDataSource[isPrivate][type] = this._createMenuEntryDataSource(type, win);
        
        return this._menuEntryDataSource[isPrivate][type];
    },
    
    getMostRecentFolder: function(type, win)
    {
        for (let entry of this.getRecentEntryDataSource(type, win).entries)
            if (! entry.hidden)
                return entry;
        
        return null;
    },
    
    preserveDownloadPrefs: function(docUri, obj, func)
    {
        let prefsBackup = this.getDownloadPrefs(docUri);
        
        try
        {
            return func.call(obj);                      
        }
        finally
        {
            this.setDownloadPrefs(prefsBackup);
        }
    },
    
    getDownloadPrefs: function(docUri, privacyContext)
    {
        var prefs = Services.prefs.getBranch("browser.download.");
        
        var downloadPrefs = {};
        downloadPrefs.useDownloadDir = prefs.getBoolPref("useDownloadDir");
        downloadPrefs.folderList = prefs.getIntPref("folderList");
        
        try { downloadPrefs.dir = prefs.getComplexValue("dir", Ci.nsIFile); }
        catch (e) { /* ignore */ }
        
        try { downloadPrefs.lastDir = prefs.getComplexValue("lastDir", Ci.nsIFile); }
        catch (e) { /* ignore */ }
        
        // last download dir is remembered per site since ff7
        var lastDownloadDir = {uri: docUri, file: this._getLastDownloadDir(docUri, privacyContext)};
        
        return {downloadPrefs: downloadPrefs, lastDownloadDir: lastDownloadDir};
    },
    
    _getLastDownloadDir: function(docUri, privacyContext)
    {/* desativando. Por que downloads via SFT não deveriam alterar a última pasta de download daquele domínio? Além do mais, isso está bugado desde o bug 886907, ao invés de get deveria ser getByDomainAndName que é async.
        // get last download dir manually instead of using gDownloadLastDir.getFile()
        // because getFile() defaults to default last download dir if per-site dir doesn't exists
        
        try
        {
            let savePerSite = true;
            try { savePerSite = Services.prefs.getBoolPref("browser.download.lastDir.savePerSite"); }
            catch (e) {}
            
            if (savePerSite)
            {
                let lastDir = Cc["@mozilla.org/content-pref/service;1"].getService(Ci.nsIContentPrefService2).get(docUri, "browser.download.lastDir", privacyContext); // não é .get, é .getByDomainAndName
                if (lastDir)
                {
                     var lastDirFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsIFile);
                     lastDirFile.initWithPath(lastDir);
                     return lastDirFile;
                }
            }
        }
        catch (e) {}*/
        
        return null;
    },
    
    setDownloadPrefs: function({downloadPrefs, lastDownloadDir}, privacyContext)
    {
        var prefs = Services.prefs.getBranch("browser.download.");
        
        // in ff2, donwload manager window listens to pref changes, so the order here is important
        if (downloadPrefs.dir)
        {
            prefs.setComplexValue("dir", Ci.nsIFile, downloadPrefs.dir);
            prefs.setIntPref("folderList", downloadPrefs.folderList);
            prefs.setBoolPref("useDownloadDir", downloadPrefs.useDownloadDir);
        }
        else
        {
            prefs.setBoolPref("useDownloadDir", downloadPrefs.useDownloadDir);
            prefs.setIntPref("folderList", downloadPrefs.folderList);
            prefs.clearUserPref("dir");
        }
        
        if (downloadPrefs.lastDir)
            prefs.setComplexValue("lastDir", Ci.nsIFile, downloadPrefs.lastDir);
        else
            prefs.clearUserPref("lastDir");
        
        // reset per-site download dir
        // can't use gDownloadLastDir.setFile() because it doesn't support clearing the pref (and it might default to default pref)
        if (lastDownloadDir?.uri.asciiHost) {
            let contentPref = Cc["@mozilla.org/content-pref/service;1"].getService(Ci.nsIContentPrefService2);
            if (lastDownloadDir.file)
                contentPref.set(lastDownloadDir.uri.asciiHost, "browser.download.lastDir", lastDownloadDir.file.path, privacyContext);
            else
                contentPref.removeByDomainAndName(lastDownloadDir.uri.asciiHost, "browser.download.lastDir", privacyContext);
        }
    },
    
    openPrefWindow: function sft_openPrefWindow(pane)
    {
        var paneId = "prefpane_" + pane;
        
        var windowUrl = "chrome://savefileto/content/prefwindow/prefwindow.xhtml";
        var windowName = "savefileto_prefwindow";
        var win = Services.wm.getMostRecentWindow(windowName);
        
        if (win)
        {
            if (paneId)
            {
                var doc = win.document;
                var prefWindow = doc.documentElement;
                prefWindow.showPane(doc.getElementById(paneId));
            }
            
            win.focus();
        }
        else
        {
            var features = "chrome,titlebar,toolbar,centerscreen,dialog=no";
            var args = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
            var variant = Cc["@mozilla.org/variant;1"].createInstance(Ci.nsIWritableVariant);
            variant.setFromVariant(paneId);
            args.appendElement(variant, false);
            Services.ww.openWindow(null, windowUrl, windowName, features, args);
        }
    },
    
    cleanUpPrivateBrowsing: function()
    {
        var isPrivate = true;
        delete this._recentEntryDataSource[isPrivate];
        delete this._menuEntryDataSource[isPrivate];
    },
    
    cleanUp: function(force)
    {
        var prefs = Services.prefs.getBranch(EXT_PREF_BRANCH);
        if (! (force || prefs.getBoolPref("clearRecent")))
            return;
        
        for (var type of TYPES)
            try
            {
                // clear normal recent folders history
                prefs.clearUserPref(type + ".recent");
                
                // clear private browsing recent folders history
                var isPrivate = true;
                if ((isPrivate in this._recentEntryDataSource) && (type in this._recentEntryDataSource[isPrivate]))
                    this._recentEntryDataSource[isPrivate][type].entries = [];
            }
            catch (e) { /* ingore */ }
    }
}


/**
 * Read-only data source (basically a view) of recent folder entries
 * that handles ordering.
 */
function SortedRecentEntryDataSource(dataSource, recentOrder)
// extends DecoratingDataSource
{
    DecoratingDataSource.call(this, dataSource);
    
    this.recentOrder = recentOrder;
    
    let self = this;
    this.recentOrder.addListener(function() { self.notifyListeners(self); });
}

SortedRecentEntryDataSource.prototype =
{
    get entries()
    {
        switch (this.recentOrder.value)
        {
            case "name":
                return this.dataSource.entries.slice().sort(compare);
            case "lastUsed":  // underlying ds is already in lastUsed order
            default:
                return this.dataSource.entries;
        }
    },
    
    set entries(entries)
    {
        throw new Error("data source is read-only");
    },
}

extend(SortedRecentEntryDataSource, DecoratingDataSource);


/**
 * @param glob.value {string} filename glob
 */
function FileNameFilter(glob)
{
    this.glob = glob;
    
    var self = this;
    this.glob.addListener(function() { delete self._re; });
}

FileNameFilter.prototype =
{
    get re()
    {
        if (! this._re)
            try
            {
                // glob -> regexp
                var glob = this.glob.value.trim();
                if (glob)
                {
                    let pattern = glob.replace(/\./g, "\\.").replace(/\*/g, ".*").replace(/\?/g, ".");
                    this._re = new RegExp("^(?:" + pattern + ")$");
                }
            }
            catch (e)
            {
                Cu.reportError(e);
            }
        
        return this._re;
    },
    
    filter: function FNF_filter(fileName)
    {
        var re = this.re;
        return (re ? re.test(fileName) : false);
    }
}


// nsIObserver
var observer =
{
    QueryInterface: (XPCOMUtils.generateQI || ChromeUtils.generateQI)([
        Ci.nsIObserver,
        Ci.nsISupportsWeakReference]),
    
    observe: function(subject, topic, data)
    {
        if (topic == "last-pb-context-exited")
            savefileto.cleanUpPrivateBrowsing();
        else if (topic == "quit-application")
            savefileto.cleanUp();
    }
}

Services.obs.addObserver(observer, "last-pb-context-exited", true);
Services.obs.addObserver(observer, "quit-application", true);
